/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef DXC_CONNECTOR_IMP_HPP
#define DXC_CONNECTOR_IMP_HPP

#include <string>
#include <fstream>
#include <vector>

#ifdef _WIN32
#include <Pthreads-w32/pthread.h>
#else
#include <pthread.h>
#endif

#include <Connection/DxcConnector.hpp>
#include <DxcCallback.hpp>
#include <DxcDataTypes.hpp>

namespace Dxc
{
class DXCAPI_EXPORT DxcConnectorImp : public DxcConnector
{
public:
    DxcConnectorImp(DxcCallback*, const std::string& logFilePrefix = "");
    virtual ~DxcConnectorImp();

    virtual bool sendMessage(const DxcData *) = 0;

protected:
    virtual void acceptConnection(std::string source, int port, int *sockp);
    virtual void acceptConnection(int port, int *sockp);
    virtual int connect(std::string source, std::string host, int port, int *sockp);
    virtual int connect(std::string host, int port, int *sockp);
    virtual bool transmit(int, const DxcData *);
    static bool socketWriteReady(int socket);
    static bool socketReadReady(int socket);
    static void closeSocket(int socket);

    struct ReceiveParam
    {
        DxcCallback * cb;
        std::string * source;
        int* sockfdp;
        bool* shutdown;
        pthread_mutex_t* parsingMutex;
    };

    static void * receive(void * receiveParam); // should be static and take (void *)
    // in order to be called by spawning thread
    static void * acceptAndReceive(void * receiveParam);

private:
    std::vector<pthread_t *> recvThreads_;
    int listenfd_;
    bool shutdown_;
    DxcCallback * callback_;
    std::ofstream* logFile_;
    // mutex to prevent long messages from arriving later
    // than subsequent shorter messages
    pthread_mutex_t parsingMutex_;
};
}

#endif
