/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef FAULT_HPP
#define FAULT_HPP

#include <iostream>
#include <string>
#include <map>
#include <set>

#include <DxcDefs.hpp>
#include <DataTypes/Value.hpp>

namespace Dxc
{

/**
 * @brief Representation of a component fault, with parameters.
 */
class DXCAPI_EXPORT Fault
{
public:
    typedef std::map<std::string, const Value*> ParameterMap;

    Fault(const std::string& componentID, const std::string& modeID);
    Fault(const std::string& componentID, const std::string& modeID, const ParameterMap& parameters);

    std::string getComponentID() const;
    std::string getModeID() const;
    const ParameterMap* getParameters() const;
    void setParameters(const ParameterMap&);

    /**
     * @brief Print the value to an output stream.
     */
    std::ostream &put(std::ostream &) const;
    bool operator==(const Dxc::Fault& f1) const;
    bool operator<(const Dxc::Fault& f1) const;

    Fault(const Fault&);
    Fault& operator=(const Fault&);
    ~Fault();

private:
    std::string componentID_;
    std::string modeID_;
    ParameterMap parameters_;
};

typedef DXCAPI_EXPORT std::set<Fault> fSet;

/**
 * @brief A set of Faults.
 */
class DXCAPI_EXPORT FaultSet : public fSet
{
public:
//    bool operator<(const FaultSet&) const;
//    bool operator==(const FaultSet&) const;
    std::ostream &put(std::ostream &) const;
};

}

std::ostream DXCAPI_EXPORT & operator << (std::ostream &, const Dxc::FaultSet&);

std::ostream DXCAPI_EXPORT & operator << (std::ostream &, const Dxc::Fault&);

#endif
